@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2022  Jean-Philippe Lang
+# Copyright (C) 2006-2023  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -89,7 +89,7 @@ class AttachmentsController < ApplicationController
           tbnail,
           :filename => filename_for_content_disposition(@attachment.filename),
           :type => detect_content_type(@attachment, true),
-          :disposition => 'inline')
+          :disposition => 'attachment')
       end
     else
       # No thumbnail for the attachment or thumbnail could not be created
@@ -219,19 +219,10 @@ class AttachmentsController < ApplicationController
   end
 
   def find_container
-    klass =
-      begin
-        params[:object_type].to_s.singularize.classify.constantize
-      rescue
-        nil
-      end
-    unless klass && (klass.reflect_on_association(:attachments) || klass.method_defined?(:attachments))
-      render_404
-      return
-    end
-
+    # object_type is constrained to valid values in routes
+    klass = params[:object_type].to_s.singularize.classify.constantize
     @container = klass.find(params[:object_id])
-    if @container.respond_to?(:visible?) && !@container.visible?
+    unless @container.visible?
       render_403
       return
     end
@@ -248,11 +239,28 @@ class AttachmentsController < ApplicationController
     if @attachments.sum(&:filesize) > bulk_download_max_size
       flash[:error] = l(:error_bulk_download_size_too_big,
                         :max_size => number_to_human_size(bulk_download_max_size.to_i))
-      redirect_to back_url
+      redirect_back_or_default(container_url, referer: true)
       return
     end
   end
 
+  def container_url
+    case @container
+    when Message
+      url_for(@container.event_url)
+    when Project
+      # project attachments are listed in the files view
+      project_files_url(@container)
+    when Version
+      # version attachments are listed in its project's files view
+      project_files_url(@container.project)
+    when WikiPage
+      project_wiki_page_url @container.wiki.project, @container
+    else
+      url_for(@container)
+    end
+  end
+
   # Checks that the file exists and is readable
   def file_readable
     if @attachment.readable?
@@ -313,4 +321,9 @@ class AttachmentsController < ApplicationController
       request.raw_post
     end
   end
+
+  def send_file(path, options={})
+    headers['content-security-policy'] = "default-src 'none'; style-src 'unsafe-inline'; sandbox"
+    super
+  end
 end
